#!/usr/bin/env bash
#set -x
shopt -s nullglob
\rm *_global_*.nc
\rm *_badcells_*.nc

for f in zos*_gn_*.nc  ; do
    echo '----------------------------------------------------------------------------------'
    echo $f
    echo '----------------------------------------------------------------------------------'
    f_allcells=${f/_gn_/_global_allcells_}
    f_goodcells=${f/_gn_/_global_goodcells_}
    f_difference=${f/_gn_/_global_difference_}
    f_badcells=${f/_gn_/_gn_badcells_}
    cdo remapcon,global_1 "$f" "$f_allcells"
    ierr="$?"
    if [[ $ierr -eq 1 ]] ; then
        echo 'Failed remapcon with all cells.'
        \rm ${f_allcells}
    fi
    echo 'Getting bad cells...'
    bad_cells=$(cdo -v verifygrid "$f" | \
                grep -v 'clockwise' | \
                grep -o -E 'cell.no.[[:digit:]]+' | \
                grep -o -E '[[:digit:]]+' | \
                sort -n | uniq | \
                tr '\n' ',')
    bad_cells=${bad_cells%?} # trim extra comma from end of string
    if [[ -n $bad_cells ]] ; then
        echo 'Bad cells: ' "$bad_cells"
        nbad=$(echo "$bad_cells" | tr ',' ' ' | wc -w)
        echo 'Total number bad cells: '"$nbad"
        cdo selgridcell,"$bad_cells" "$f" "$f_badcells"  
        [[ $? -eq 0 ]] && echo 'Bad cells written to <'"$f_badcells"'>.'
        echo 'The value of zos printed below, if not NaN, is he number of cells identified as bad, but are not NaN. I guess these are the duplicates that should have been kept...'
        cdo output -fldsum -expr,'zos = (zos == zos) ? 1 : (zos/0.)' "$f_badcells"
        echo 'Trying remapcon without bad cells...'
        cdo remapcon,global_1 -delgridcell,"$bad_cells" "$f" "$f_goodcells"
	    ierr="$?"
        if [[ $ierr -eq 0 && -f $f_allcells && -f $f_goodcells ]] ; then
            echo 'remapcon successful. Calculating difference...'
            cdo sub "$f_goodcells" "$f_allcells" "$f_difference"
        fi
    else
        echo 'No bad cells detected.'
    fi
done
